/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-10-17 16:31:14.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.print.common.jpa.OrderBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SearchBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.print.common.jpa.SqlWrapper;
import cn.com.qmth.examcloud.core.print.common.utils.Check;
import cn.com.qmth.examcloud.core.print.entity.PrintingProject;
import cn.com.qmth.examcloud.core.print.repository.PrintingProjectRepository;
import cn.com.qmth.examcloud.core.print.service.PrintingProjectService;
import cn.com.qmth.examcloud.core.print.service.StatisticService;
import cn.com.qmth.examcloud.core.print.service.bean.common.ExamInfo;
import cn.com.qmth.examcloud.core.print.service.bean.common.OrgInfo;
import cn.com.qmth.examcloud.core.print.service.bean.printingproject.PrintingProjectConvert;
import cn.com.qmth.examcloud.core.print.service.bean.printingproject.PrintingProjectInfo;
import cn.com.qmth.examcloud.core.print.service.bean.printingproject.PrintingProjectLessInfo;
import cn.com.qmth.examcloud.core.print.service.bean.printingproject.PrintingProjectQuery;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

import static cn.com.qmth.examcloud.core.print.common.Constants.SYS_CODE_500;

/**
 * 印刷项目相关接口
 *
 * @author: fengdesheng
 * @since: 2018/10/17
 */
@Service
public class PrintingProjectServiceImpl implements PrintingProjectService {
    private static final Logger log = LoggerFactory.getLogger(PrintingProjectServiceImpl.class);
    @Autowired
    private PrintingProjectRepository printingProjectRepository;
    @Autowired
    private StatisticService statisticService;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Override
    public Page<PrintingProjectInfo> getPrintingProjectList(PrintingProjectQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        //查询条件
        SearchBuilder searches = new SearchBuilder();
        if (query.getOrgId() != null) {
            searches.eq("orgId", query.getOrgId());
        }
        if (query.getExamId() != null) {
            searches.eq("examId", query.getExamId());
        }
        if (query.getPmId() != null) {
            searches.eq("pmId", query.getPmId());
        }
        if (query.getSupplierId() != null) {
            searches.eq("supplierId", query.getSupplierId());
        }
        Specification<PrintingProject> spec = SpecUtils.buildSearchers(PrintingProject.class, searches.build());
        //排序条件
        OrderBuilder orders = new OrderBuilder().desc("id");
        //分页条件
        Pageable pageable = SpecUtils.buildPageable(query.getPageNo(), query.getPageSize(), orders.build());
        Page<PrintingProject> page = printingProjectRepository.findAll(spec, pageable);
        return PrintingProjectConvert.ofPage(page);
    }

    @Override
    public PrintingProjectInfo getPrintingProjectById(Long id) {
        Check.isNull(id, "印刷项目ID不能为空！");
        Optional<PrintingProject> optional = printingProjectRepository.findById(id);
        if (!optional.isPresent()) {
            throw new StatusException(SYS_CODE_500, "印刷项目信息不存在！");
        }
        return PrintingProjectConvert.of(optional.get());
    }

    @Override
    public PrintingProject getPrintingProjectByOrgIdAndExamId(Long orgId, Long examId) {
        SearchBuilder searches = new SearchBuilder()
                .eq("orgId", orgId)
                .eq("examId", examId);
        Specification<PrintingProject> spec = SpecUtils.buildSearchers(PrintingProject.class, searches.build());
        Optional<PrintingProject> optional = printingProjectRepository.findOne(spec);
        if (optional.isPresent()) {
            return optional.get();
        }
        return null;
    }

    @Override
    public void updatePrintingProject(PrintingProjectInfo info) {
        Check.isNull(info.getId(), "印刷项目ID不能为空！");
        Check.isNull(info.getSupplierId(), "供应商ID不能为空！");
        Check.isBlank(info.getSupplierName(), "供应商名称不能为空！");
        Check.isNull(info.getPmId(), "项目经理ID不能为空！");
        Check.isBlank(info.getPmName(), "项目经理名称不能为空！");
        Check.isNull(info.getPrepareStartTime(), "准备开始时间不能为空！");
        Check.isNull(info.getPrepareEndTime(), "准备结束时间不能为空！");
        Check.isNull(info.getPrintStartTime(), "印刷开始时间不能为空！");
        Check.isNull(info.getPrintEndTime(), "印刷结束时间不能为空！");
        Check.isNull(info.getMailStartTime(), "邮寄开始时间不能为空！");
        Check.isNull(info.getMailEndTime(), "邮寄结束时间不能为空！");

        Optional<PrintingProject> optional = printingProjectRepository.findById(info.getId());
        if (!optional.isPresent()) {
            throw new StatusException(SYS_CODE_500, "印刷项目信息不存在！");
        }
        PrintingProject entity = optional.get();
        entity.setSupplierId(info.getSupplierId());
        entity.setSupplierName(info.getSupplierName());
        entity.setPmId(info.getPmId());
        entity.setPmName(info.getPmName());
        entity.setPrepareStartTime(info.getPrepareStartTime());
        entity.setPrepareEndTime(info.getPrepareEndTime());
        entity.setPrintStartTime(info.getPrintStartTime());
        entity.setPrintEndTime(info.getPrintEndTime());
        entity.setMailStartTime(info.getMailStartTime());
        entity.setMailEndTime(info.getMailEndTime());
        printingProjectRepository.save(entity);
    }

    @Override
    public void syncPrintingProject(ExamInfo examInfo) {
        Check.isNull(examInfo.getOrgId(), "学校ID不能为空！");
        Check.isBlank(examInfo.getOrgName(), "学校名称不能为空！");
        Check.isNull(examInfo.getExamId(), "考试ID不能为空！");
        Check.isBlank(examInfo.getExamName(), "考试名称不能为空！");

        SearchBuilder searches = new SearchBuilder()
                .eq("orgId", examInfo.getOrgId())
                .eq("examId", examInfo.getExamId());
        Specification<PrintingProject> spec = SpecUtils.buildSearchers(PrintingProject.class, searches.build());

        //某学校某考试仅当作一个项目
        Optional<PrintingProject> optional = printingProjectRepository.findOne(spec);
        if (optional.isPresent()) {
            //更新学校名称、考试名称等信息
            PrintingProject project = optional.get();
            project.setOrgName(examInfo.getOrgName());
            project.setExamName(examInfo.getExamName());
            printingProjectRepository.save(project);
            return;
        }

        //新增印刷项目信息
        PrintingProject project = new PrintingProject();
        project.setOrgId(examInfo.getOrgId());
        project.setOrgName(examInfo.getOrgName());
        project.setExamId(examInfo.getExamId());
        project.setExamName(examInfo.getExamName());
        project.setCompleted(false);
        printingProjectRepository.save(project);
    }

    @Override
    public void initAllPrintingProject() {
        //获取所有"传统"考试列表
        log.debug("initAllPrintingProject...");
        List<ExamInfo> list = statisticService.findTraditionExams();
        if (list != null && !list.isEmpty()) {
            for (ExamInfo info : list) {
                syncPrintingProject(info);
            }
        }
    }

    @Override
    public void syncOrgNameByOrgId(Long orgId, String orgName) {
        if (orgId == null || StringUtils.isBlank(orgName)) {
            return;
        }
        printingProjectRepository.updateOrgNameByOrgId(orgId, orgName);
    }

    @Override
    public void syncExamNameByExamId(Long examId, String examName) {
        if (examId == null || StringUtils.isBlank(examName)) {
            return;
        }
        printingProjectRepository.updateExamNameByExamId(examId, examName);
    }

    @Override
    public void syncPmNameByPmId(Long pmId, String pmName) {
        if (pmId == null || StringUtils.isBlank(pmName)) {
            return;
        }
        printingProjectRepository.updatePmNameByPmId(pmId, pmName);
    }

    @Override
    public void syncSupplierNameBySupplierId(Long supplierId, String supplierName) {
        if (supplierId == null || StringUtils.isBlank(supplierName)) {
            return;
        }
        printingProjectRepository.updateSupplierNameBySupplierId(supplierId, supplierName);
    }

    @Override
    public List<PrintingProjectLessInfo> getPrintingProjectLessInfoList() {
        SqlWrapper sql = new SqlWrapper().select("id as projectId,org_id,exam_id").from("ec_prt_project");
        return jdbcTemplate.query(sql.build(), new BeanPropertyRowMapper(PrintingProjectLessInfo.class));
    }

    @Override
    public List<OrgInfo> getOrgList(Long pmId, Long supplierId) {
        SqlWrapper sql = new SqlWrapper().select("org_id,org_name").from("ec_prt_project").where().append("1=1");
        if (pmId != null) {
            sql.and().eq("pm_id", pmId);
        }
        if (supplierId != null) {
            sql.and().eq("supplier_id", supplierId);
        }
        sql.groupBy("org_id");
        return jdbcTemplate.query(sql.build(), new BeanPropertyRowMapper(OrgInfo.class));
    }

    @Override
    public List<ExamInfo> getExamList(Long orgId, Long pmId, Long supplierId) {
        SqlWrapper sql = new SqlWrapper().select("org_id,org_name,exam_id,exam_name").from("ec_prt_project").where().append("1=1");
        if (orgId != null) {
            sql.and().eq("org_id", orgId);
        }
        if (pmId != null) {
            sql.and().eq("pm_id", pmId);
        }
        if (supplierId != null) {
            sql.and().eq("supplier_id", supplierId);
        }
        sql.groupBy("exam_id");
        return jdbcTemplate.query(sql.build(), new BeanPropertyRowMapper(ExamInfo.class));
    }

}