/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-10-26 18:28:34.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.print.common.utils.Check;
import cn.com.qmth.examcloud.core.print.common.utils.DateUtils;
import cn.com.qmth.examcloud.core.print.entity.PrintingProject;
import cn.com.qmth.examcloud.core.print.entity.ProjectBackupSetting;
import cn.com.qmth.examcloud.core.print.entity.ProjectStatistic;
import cn.com.qmth.examcloud.core.print.enums.BackupGroupType;
import cn.com.qmth.examcloud.core.print.repository.ProjectBackupSettingRepository;
import cn.com.qmth.examcloud.core.print.repository.ProjectStatisticRepository;
import cn.com.qmth.examcloud.core.print.service.CourseStatisticService;
import cn.com.qmth.examcloud.core.print.service.PrintingProjectService;
import cn.com.qmth.examcloud.core.print.service.PrintingProjectStatisticService;
import cn.com.qmth.examcloud.core.print.service.StatisticService;
import cn.com.qmth.examcloud.core.print.service.bean.common.ExamCourseInfo;
import cn.com.qmth.examcloud.core.print.service.bean.common.RefreshInfo;
import cn.com.qmth.examcloud.core.print.service.bean.coursestatistic.CourseStatisticLessInfo;
import cn.com.qmth.examcloud.core.print.service.bean.coursestatistic.CourseStatisticRefreshReq;
import cn.com.qmth.examcloud.core.print.service.bean.printingproject.PrintingProjectLessInfo;
import cn.com.qmth.examcloud.core.print.service.bean.printingprojectstatistic.PrintingProjectStatisticConvert;
import cn.com.qmth.examcloud.core.print.service.bean.printingprojectstatistic.PrintingProjectStatisticInfo;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

import static cn.com.qmth.examcloud.core.print.common.Constants.SYS_CODE_500;

/**
 * @author: fengdesheng
 * @since: 2018/10/26
 */
@Service
public class PrintingProjectStatisticServiceImpl implements PrintingProjectStatisticService {
    private static final Logger log = LoggerFactory.getLogger(PrintingProjectStatisticServiceImpl.class);
    @Autowired
    private ProjectStatisticRepository projectStatisticRepository;
    @Autowired
    private PrintingProjectService printingProjectService;
    @Autowired
    private CourseStatisticService courseStatisticService;
    @Autowired
    private ProjectBackupSettingRepository projectBackupSettingRepository;
    @Autowired
    private StatisticService statisticService;

    @Override
    public PrintingProjectStatisticInfo getPrintingProjectStatistic(Long orgId, Long examId) {
        Check.isNull(orgId, "学校ID不能为空！");
        Check.isNull(examId, "考试ID不能为空！");

        PrintingProject project = printingProjectService.getPrintingProjectByOrgIdAndExamId(orgId, examId);
        if (project == null) {
            throw new StatusException(SYS_CODE_500, "当前项目不存在！");
        }

        ProjectStatistic statistic = projectStatisticRepository.getProjectStatisticByProjectId(project.getId());
        return PrintingProjectStatisticConvert.of(project, statistic);
    }

    @Override
    public void refreshPrintingProjectStatistic(Long orgId, Long examId) {
        Check.isNull(orgId, "学校ID不能为空！");
        Check.isNull(examId, "考试ID不能为空！");

        PrintingProject project = printingProjectService.getPrintingProjectByOrgIdAndExamId(orgId, examId);
        if (project == null) {
            throw new StatusException(SYS_CODE_500, "当前项目不存在！");
        }

        if (RefreshInfo.projectRefreshing) {
            throw new StatusException(SYS_CODE_500, "尚有项目统计任务正在刷新中，请稍后再刷新！");
        }

        //目前限定仅一个刷新任务可执行
        RefreshInfo.projectRefreshing = true;
        Date start = new Date();
        log.debug("refreshPrintingProjectStatistic start...");
        try {
            //刷新课程统计
            courseStatisticService.refreshCourseStatistic(new CourseStatisticRefreshReq(orgId, examId));

            //执行项目统计
            this.doStatistic(Lists.newArrayList(new PrintingProjectLessInfo(orgId, examId, project.getId())));
        } catch (StatusException e) {
            throw e;//状态异常则直接抛出
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        } finally {
            RefreshInfo.projectRefreshing = false;
            log.debug("refreshPrintingProjectStatistic finished..." + DateUtils.diff(start, new Date()));
        }
    }

    @Override
    public void initAllPrintingProjectStatistic() {
        log.debug("initAllPrintingProjectStatistic...");

        //只处理已有的印刷项目
        List<PrintingProjectLessInfo> projects = printingProjectService.getPrintingProjectLessInfoList();

        this.doStatistic(projects);
    }

    private void doStatistic(List<PrintingProjectLessInfo> projects) {
        if (projects == null || projects.isEmpty()) {
            return;
        }

        List<ProjectStatistic> statistics = new ArrayList<>();
        for (PrintingProjectLessInfo project : projects) {
            Date start = new Date();

            //逐个考试来统计印刷项目的汇总结果，同时多个考试会有查询效率问题(涉及表较多且数据高达百万或以上)
            ProjectStatistic statistic = projectStatisticRepository.getProjectStatisticByProjectId(project.getProjectId());
            if (statistic == null) {
                statistic = new ProjectStatistic();
                statistic.setProjectId(project.getProjectId());
            }

            //获取某考试的课程和试卷类型列表
            List<ExamCourseInfo> coursePaperTypes = statisticService.findExamCourseAndPaperTypes(project.getOrgId(), project.getExamId());

            //获取某考试的试卷袋列表
            List<String> packageCodes = statisticService.findExamPackageCodes(project.getOrgId(), project.getExamId());

            //获取某考试的所有课程的试卷页数和考生人数信息列表
            List<CourseStatisticLessInfo> courseStatistics = courseStatisticService.getCourseStatisticLessInfoList(project.getOrgId(), project.getExamId());

            //获取某个项目的备份设置信息
            ProjectBackupSetting backupSetting = projectBackupSettingRepository.findByProjectId(project.getProjectId());

            //人科次(考生的数量)
            final int totalStudent = statisticService.countExamTotalStudentByExamId(project.getExamId());

            //课程数量(课程代码的数量)
            final int totalCourse = statisticService.countExamTotalCourse(coursePaperTypes);
            log.debug("常规统计耗时：" + DateUtils.diff(start, new Date()));

            //试卷数量(课程代码 + 试卷类型的数量)
            final int totalPaper = coursePaperTypes.size();

            //试卷袋数量
            final int totalPkg = packageCodes.size();

            //常规-A3数量(总试卷A3数 + 总答题卡A3数)
            final int normalA3 = this.parseNormalA3(courseStatistics, totalStudent);

            //常规-A4数量(试卷袋数 * 4)
            final int normalA4 = totalPkg * 4;

            //统计备份A3、A4数
            Map<String, Integer> backupMaps = this.parseBackupA3A4(backupSetting, packageCodes, project.getExamId());

            //备份-A3数量(每袋冗余数 + 单独备份袋数)
            final int backupA3 = backupMaps.get("backupA3");

            //备份-A4数量(∑n (学习中心或考点) * 2)
            final int backupA4 = backupMaps.get("backupA4");

            statistic.setTotalStudent(totalStudent);
            statistic.setTotalCourse(totalCourse);
            statistic.setTotalPaper(totalPaper);
            statistic.setTotalPkg(totalPkg);
            statistic.setNormalA3(normalA3);
            statistic.setNormalA4(normalA4);
            statistic.setBackupA3(backupA3);
            statistic.setBackupA4(backupA4);
            statistics.add(statistic);
        }

        //批量保存
        projectStatisticRepository.saveAll(statistics);
    }

    private int parseNormalA3(List<CourseStatisticLessInfo> courseStatistics, int totalStudent) {
        int sumPaperA3 = 0;
        if (courseStatistics != null && !courseStatistics.isEmpty()) {
            for (CourseStatisticLessInfo courseStatistic : courseStatistics) {
                sumPaperA3 += courseStatistic.getMultiplicationValue();
            }
        }
        //∑n 试卷(试卷P数 * 当前试卷人科次数) + (总科次人数) * 2
        return sumPaperA3 + (totalStudent * 2);
    }

    private Map<String, Integer> parseBackupA3A4(ProjectBackupSetting backupSetting, List<String> packageCodes, Long examId) {
        Map<String, Integer> backupMaps = new HashMap<>();
        backupMaps.put("backupA3", 0);
        backupMaps.put("backupA4", 0);

        if (backupSetting == null) {
            return backupMaps;
        }

        if (!backupSetting.getNeedEachPkg() && !backupSetting.getNeedAlonePkg()) {
            return backupMaps;
        }

        int backupA3 = 0, backupA4 = 0;

        if (backupSetting.getNeedEachPkg()) {
            //备份A3 = ∑n 试卷袋(X考生 * P%备份比例) * 2
            int eachTotal = 0;
            Date start = new Date();

            Double percent = backupSetting.getEachPkgPercent();//备份比例：高于最大取最大；低于最小则取最小
            if (percent > backupSetting.getEachPkgMax()) {
                percent = backupSetting.getEachPkgMax();
            }
            if (percent < backupSetting.getEachPkgMin()) {
                percent = backupSetting.getEachPkgMin();
            }

            for (String packageCode : packageCodes) {
                int count = statisticService.countExamTotalStudentByPackageCode(examId, packageCode);
                eachTotal += Math.ceil((count * percent) / 100);
            }
            backupA3 += (eachTotal * 2);
            log.debug("每袋备份统计耗时：" + DateUtils.diff(start, new Date()));
        }

        if (backupSetting.getNeedAlonePkg()) {
            //备份A3 = ∑n 学习中心或考点(X考生 * P%备份比例) * 2
            int aloneTotal = 0;
            Date start = new Date();

            Double percent = backupSetting.getAlonePkgPercent();//备份比例：高于最大取最大；低于最小则取最小
            if (percent > backupSetting.getAlonePkgMax()) {
                percent = backupSetting.getAlonePkgMax();
            }
            if (percent < backupSetting.getAlonePkgMin()) {
                percent = backupSetting.getAlonePkgMin();
            }

            if (BackupGroupType.isLeanCenter(backupSetting.getGroupType())) {
                //计算每个学习中心的备份数
                List<Long> learnCenters = statisticService.findExamLearnCenters(examId);
                for (Long learnCenterId : learnCenters) {
                    int count = statisticService.countExamTotalStudentByLearnCenter(examId, learnCenterId);
                    aloneTotal += Math.ceil((count * percent) / 100);
                }

                //备份-A4数量：只有单独备份袋设置才计算，默认不用算
                backupA4 = learnCenters.size() * 2;
            } else {
                //计算每个考点的备份数
                List<String> siteIds = statisticService.findExamSites(examId);
                for (String siteId : siteIds) {
                    int count = statisticService.countExamTotalStudentBySite(examId, siteId);
                    aloneTotal += Math.ceil((count * percent) / 100);
                }

                //备份-A4数量：只有单独备份袋设置才计算，默认不用算
                backupA4 = siteIds.size() * 2;
            }

            backupA3 += (aloneTotal * 2);
            log.debug("单独备份袋统计耗时：" + DateUtils.diff(start, new Date()));
        }

        backupMaps.put("backupA3", backupA3);
        backupMaps.put("backupA4", backupA4);
        return backupMaps;
    }

}